/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/zephyr.h>
#include <sys/printk.h>
#include <drivers/uart.h>





static K_SEM_DEFINE(tx_sem, 0, 1); // Semaphore to signal UART data transmitted
static uint8_t* tx_data;
static size_t tx_data_length;

static void uart_irq_callback(const struct device *dev, void *user_data)
{
    ARG_UNUSED(user_data);
    static int tx_data_idx;
    if (!uart_irq_update(dev))
    {
            printk("Couldn't update IRQ\n");
            return;
    }
    if (uart_irq_tx_ready(dev) && tx_data_idx < tx_data_length)
    {
        int tx_send = MIN(CONFIG_UART_1_NRF_TX_BUFFER_SIZE, tx_data_length - tx_data_idx);
        int tx_sent = uart_fifo_fill(dev, (uint8_t *)&tx_data[tx_data_idx], tx_send);
        if (tx_sent <= 0)
        {
             printk("Error %d sending data over RS485 bus\n", tx_sent);
             return;
        }
        tx_data_idx += tx_sent;
        if (tx_data_idx == tx_data_length)
        {
            uart_irq_tx_disable(dev);
            tx_data_idx = 0;
            k_sem_give(&tx_sem);
        }
    }

 }
int uart_irq_tx(const struct device *dev, uint8_t *buf, size_t len)
{
    tx_data = buf;
    tx_data_length = len;
    uart_irq_callback_set(dev, uart_irq_callback);
    uart_irq_tx_enable(dev);
    k_sem_take(&tx_sem, K_FOREVER);
    printk("UART data transmitted\n");
    return len; 
}

void main(void)
{
	int len = 0;
    const struct device *dev = DEVICE_DT_GET(DT_NODELABEL(uart0));
    if (!dev) {
        printk("Failed to get UART binding\n");
    }

    static uint8_t message[] = "Test message\n";
    
    len = uart_irq_tx(dev, message, sizeof(message));

    printk("TX length %d\n", len);
	
	printk("Hello World! %s\n", CONFIG_BOARD);
}
